package com.hextools.disk

import android.app.Activity
import android.content.Context
import android.os.Environment
import android.os.StatFs
import android.os.storage.StorageManager
import android.os.storage.StorageVolume
import androidx.annotation.NonNull
import androidx.core.content.ContextCompat.getSystemService

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.security.AccessController.getContext

/** DiskPlugin */
class DiskPlugin: FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel : MethodChannel
    private lateinit var context: Context

    // This function returns a `list` of `map` of name and path of mounted storage volumes in device
    // ONLY WORKS FOR SDK 30 AND ABOVE
    private fun getMountedStorageVolumes(): MutableList<Map<String, String?>> {
        // Map<String, String> of `nameOfVolume` to `volumeHomePath`5
        var output = mutableListOf<Map<String, String?>>()

        val storageManager= context.getSystemService(Context.STORAGE_SERVICE) as StorageManager

        val storageVolumes = storageManager.storageVolumes

        for (storageVolume:StorageVolume in storageVolumes) {
            if (storageVolume.state != Environment.MEDIA_UNMOUNTED) {
                val name = if (storageVolume.isRemovable) "Removable Disk" else "Internal Storage"
                val path = if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.R) {
                    storageVolume.directory.toString()
                } else {
                    null
                }

                var map = mapOf<String, String?>("name" to name, "path" to path)
                output.add(map)
            }
        }

        return output
    }

    private fun getTotalBytes(path: String): Long {
        val statFs = StatFs(path)

        return statFs.totalBytes
    }

    private fun getFreeBytes(path: String): Long {
        val statFs = StatFs(path)

        return statFs.freeBytes
    }


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "disk")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getMountedStorageVolumes" -> {
                result.success(getMountedStorageVolumes())
            }

            "getTotalBytes" -> {
                val path = call.argument<String>("path")!!
                result.success(getTotalBytes(path))
            }

            "getFreeBytes" -> {
                val path = call.argument<String>("path")!!
                result.success(getFreeBytes(path))
            }

            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }

            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

}
